import QtQuick 2.0
import QtQuick.XmlListModel 2.0

import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItems
import Ubuntu.Components.Popups 0.1

import "databasemodule.js" as DB
import "databasemodule_v2.js" as DB2

Page {
    id: pageItself

    signal feedSelected(var params)
    signal feedDeleted(string feedLink)

    property var currentTask: null // Mb need task queue

    Component.onCompleted: {
        thinXmlHttpRequestWraper.downloadFinished.connect(processDownloadResult)
        helperModel.workFinished.connect(processWorkResult)

        // DB2.dropTable() only for debug.
        feedList.reloadFeeds()
    }

    function processDownloadResult(result) {
        switch(currentTask.operation)
        {
        case "append":
            if (result.error) {
                console.log("Append stage 1 ERROR: ", result.status, result.statusText)
                dispatchFailure()
            } else {
                helperModel.startFeedInfoParsing(result.xml)
            }
            break
        case "update":
            if (result.error) {
                console.log("Update stage 1 ERROR: ", result.status, result.statusText)
                dispatchFailure()
            } else {
                helperModel.startFeedInfoParsing(result.xml)
            }
            break
        default:
            console.log("processDownloadResult something wrong: ", currentTask.operation)
        }
    }

    function processWorkResult(result) { // bad name :(
        console.log("processWorkResult error", result.error)

        switch(currentTask.operation)
        {
        case "append":
            if (result.error) {
                console.log("Append stage 2 ERROR: ", result.errorString)
                dispatchFailure()
            } else {
                // we must go on, parse downloaded xml.

                DB2.setFeedStatus(currentTask.feedId, '1')
                DB2.updateFeedByXml(currentTask.feedId, result.data.link,
                                    result.data.description, result.data.title)

                feedList.reloadFeeds()

                xmlModelToParseItems.startEntriesParsing(helperModel.xml)
            }
            break
        case "update":
            if (result.error) {
                console.log("Update stage 2 ERROR: ", result.errorString)
                dispatchFailure()
            } else {
                DB2.setFeedStatus(currentTask.feedId, '1')
                DB2.updateFeedByXml(currentTask.feedId, result.data.link,
                                    result.data.description, result.data.title)

                feedList.reloadFeeds()

                xmlModelToParseItems.startEntriesParsing(helperModel.xml)
            }
            break;
        default:

        }
    }

    function dispatchFailure() {
        DB2.setFeedStatus(currentTask.feedId, '2')

        var i
        for (i = 0; i < feedListModel.count; i++) {
            if (feedListModel.get(i).id == currentTask.feedId) {
                feedListModel.setProperty(i, "status", '2')
                feedListModel.setProperty(i, "title", currentTask.source)
                feedListModel.setProperty(i, "description", "FAILURE") // TODO TEMP
                break
            }
        }

        if (i === feedListModel.count) {
            // Something gone wrong - no item in listModel
            console.log("NO ITEM IN MODEL WITH ID:", currentTask.feedId)
        }

        currentTask = null
    }

    ListView {
        id: feedList

        function reloadFeeds() {
            //feedList.model = null
            feedListModel.clear()

            var feeds = DB2.loadFeeds()

            for (var i = 0; i < feeds.rows.length; i++) {
                feedListModel.append({"source":feeds.rows.item(i).source,
                                         "title":feeds.rows.item(i).title,
                                         "link":feeds.rows.item(i).link,
                                         "description":feeds.rows.item(i).description,
                                         "status":feeds.rows.item(i).status,
                                         "id":feeds.rows.item(i).id})
            }
        }

        anchors.fill: parent
        model: ListModel {
            id: feedListModel
        }

        header: ListItems.Subtitled {
            text: i18n.tr("All news")
            subText: i18n.tr("View news from all channels")
            progression: true
            visible: feedListModel.count != 0

            onClicked: {
                pageItself.feedSelected({"isAll":true})
            }
        } // header

        delegate: ListItems.Subtitled {
            objectName: "feedItem"
            text: (model.title == "")? model.source /*+ "   " + model.status*/ : model.title /*+ "   " + model.status*/ // Debug - displaying status
            subText: (model.description != undefined)? model.description : "EMPTY"
            progression: true


            onPressAndHold: {
                feedList.currentIndex = model.index
                PopupUtils.open(popoverComponent, feedList)
            }

            onClicked: {
                console.log("Show news in feed:", model.source)
                pageItself.feedSelected({"isAll":false, "feedId":model.id})
            }
        } // delegate

    } // ListView

    /* Thin wrapper around XMLHttpRequest.
     * 0 stage. Loading raw XML data from network.
     */
    QtObject {
        id: thinXmlHttpRequestWraper

        signal downloadFinished(var result)

        function loadXml(xmlUrl) {
            xmlUrl = xmlUrl.trim()
            if (xmlUrl.indexOf('http') !== 0)
                xmlUrl = 'http://' + xmlUrl

            var doc = new XMLHttpRequest();
            doc.onreadystatechange = function() {
                if (doc.readyState === XMLHttpRequest.DONE) {
                    if (doc.status == 200 ) {
                        console.log("thinXmlHttpRequestWraper SUCCESS")
                        downloadFinished({"error" : false, "xml" : doc.responseText})
                    } else {
                        console.log("thinXmlHttpRequestWraper ERROR", doc.status, doc.statusText)
                        downloadFinished({"error" : true, "status" : doc.status,
                                             "statusText" : doc.statusText})
                    }
                }
            }

            doc.open("GET", xmlUrl);
            doc.send();
        }
    } // QtObject

    /* First stage - need to gain information about feed from XML
     * and save it to database. Next step - loading entries.
     */
    XmlListModel {
        id: helperModel

        function startFeedInfoParsing(strXml) {
            if (helperModel.xml === strXml) {
                console.log("helperModel SAME XML")
                modelReady()
            }
            else helperModel.xml = strXml
        }

        signal workFinished(var result)

        query: "/rss/channel"

        XmlRole { name: "title"; query: "title/string()" }
        XmlRole { name: "link"; query: "link/string()" }
        XmlRole { name: "description"; query: "description/string()" }

        onStatusChanged: {

            switch(status)
            {
            case XmlListModel.Error:
                console.log("helperModel", "ERROR", errorString())
                workFinished({"error" : true, "errorString" : errorString()})
                break
            case XmlListModel.Ready:
                modelReady()
                break
            case XmlListModel.Loading:
                //console.log("helperModel", "LOADING")
                break
            case XmlListModel.Null:
                //console.log("helperModel", "NULL")
                break
            }
        }

        function modelReady() {
            var firstItem
            console.log("helperModel", "READY", "COUNT", count)
            if (count === 1) {
                firstItem = helperModel.get(0)
                workFinished({"error" : false, "data" : firstItem})
            } else { // if (count === 0 ) {
                workFinished({"error" : true, "errorString" : "Count not equal to 1"})
            }
        }
    } // XmlListModel

    /* Second stage - need to parse XML and get all items from it.
     * This items will be displayed by "NewsListPage.qml"
     *
     * Very useful http://www.w3schools.com/xpath/xpath_functions.asp
     */
    XmlListModel {
        id: xmlModelToParseItems

        function startEntriesParsing(strXml) {
            if (xmlModelToParseItems.xml === strXml)
                modelReady()
            else xmlModelToParseItems.xml = strXml
        }

        query: "/rss/channel/item"

        XmlRole { name: "title"; query: "title/string()" }
        XmlRole { name: "content"; query: "*[name()='content:encoded']/string()" }
        XmlRole { name: "link"; query: "link/string()" }
        XmlRole { name: "description"; query: "description/string()" }
        XmlRole { name: "pubDate"; query: "pubDate/string()" }
        XmlRole { name: "guid"; query: "guid/string()" }

        onStatusChanged: {

            switch(status)
            {
            case XmlListModel.Error:
                console.log("xmlModelToParseItems", "ERROR", errorString())
                break
            case XmlListModel.Ready:
            {
                modelReady()
            }
            break
            case XmlListModel.Loading:
                //console.log("xmlModelToParseItems", "LOADING")
                break
            case XmlListModel.Null:
                //console.log("xmlModelToParseItems", "NULL")
                break
            }
        }

        function modelReady() {
            console.time("APPEND TO MODEL")
            var entry
            console.log("xmlModelToParseItems", "READY", "COUNT", count)
            for (var i = 0; i < count; i++) {
                entry = xmlModelToParseItems.get(i)
                DB2.addArticle(entry.title, entry.content, entry.link, entry.description,
                               entry.pubDate, entry.guid, currentTask.feedId)
            }
            currentTask = null
            console.timeEnd("APPEND TO MODEL")
        }
    } // XmlListModel

    tools: ToolbarActions {
        Action {
            text: i18n.tr("Add feed")
            iconSource: Qt.resolvedUrl("avatar.png")
            onTriggered: {
                if (currentTask != null) {
                    console.log("TASK ALREADY RUNNING") // TODO msg to user or use queue.
                    return
                }

                PopupUtils.open(popoverSheetComponent, tools)
            }
        }
    }

    Component {
        id: popoverComponent

        /* This new popover works like context menu.
         * Actions can be reused in toolbar.
         */
        ActionSelectionPopover {
            id: popover
            actions: ActionList {
                id: popoverActionsList
                Action {
                    objectName: "updateFeed"
                    id: nameActionUpdate
                    text: i18n.tr("Update")
                    onTriggered: {
                        PopupUtils.close(popover)
                        var data = feedListModel.get(feedList.currentIndex)
                        var feedUrl = data.source
                        console.log("UPDATE STARTED FOR FEED:", feedUrl, data.id)
                        pageItself.currentTask = {"operation" : "update", "source" : feedUrl, "feedId" : data.id}
                        thinXmlHttpRequestWraper.loadXml(feedUrl)
                    }
                }

                Action {
                    text: i18n.tr("Edit")
                    onTriggered: {
                        PopupUtils.close(popover)
                        var data = feedListModel.get(feedList.currentIndex)
                        console.log("START EDIT ", data.source, data.status)
                        if (data.status == '0') {
                            console.log("EDIT ABORT")
                            return
                        }

                        PopupUtils.open(popoverSheetComponent, tools, {"isForEdit":true, "itemData" : data})
                    }
                }

                Action {
                    text: i18n.tr("Open site")
                    onTriggered: {
                        PopupUtils.close(popover)
                        Qt.openUrlExternally(feedListModel.get(feedList.currentIndex).link)
                    }
                }

                Action {
                    objectName: "removeFeed"
                    text: i18n.tr("Remove")
                    onTriggered: {
                        PopupUtils.close(popover)
                        var id = feedListModel.get(feedList.currentIndex).id
                        var dbResult = DB2.deleteFeed(id)
                        if (dbResult.rowsAffected > 0) {
                            DB2.clearArticles(id)
                            feedListModel.remove(feedList.currentIndex)
                        }
                        // feedDeleted(id) // TODO think.
                    }
                }

                Action {
                    text: i18n.tr("Remove all")
                    onTriggered: {
                        PopupUtils.close(popover)
                        DB2.dropTable()
                        feedListModel.clear()
                    }
                }
            } // ActionList
        } // ActionSelectionPopover
    } // Popover component

    /* Modal sheet for appending feeds.
     * Demo of this: https://ubuntu.mybalsamiq.com/projects/ubuntuphonecoreapps/RSS_reader_step1.1
     */
    Component {
        id: popoverSheetComponent

        ComposerSheet {
            id: appendSheet

            property bool isForEdit: false
            property var itemData: null
            // property alias showOnMainScreen: swShowOnMainscreen.checked

            Component.onCompleted: {
                if (itemData) {
                    console.log("popoverSheetComponent", itemData.status)
                    if (itemData.status == '1') { // Can edit only title.
                        tfFeedUrl.text = itemData.title
                    } else if (itemData.status == '2') { // Must edit source - if it's wrong.
                        tfFeedUrl.text = itemData.source
                    }
                }

                tfFeedUrl.forceActiveFocus()
                tfFeedUrl.cursorPosition = tfFeedUrl.text.length
            }

            onConfirmClicked: {
                if (isForEdit) {
                    if (itemData.status == '1') {
                        if (tfFeedUrl.text != "") {
                            itemData.title = tfFeedUrl.text
                            DB2.updateFeedByUser(itemData.id, itemData.title, itemData.source)
                        }
                    } else if (itemData.status == '2') {
                        if (tfFeedUrl.text != "") {
                            itemData.source = tfFeedUrl.text
                            DB2.updateFeedByUser(itemData.id, itemData.source, itemData.source)

                            /* Update after editing source.
                             */
                            var feedUrl = itemData.source
                            // console.log("EDIT CHANGE:", feedUrl, itemData.id)
                            pageItself.currentTask = {"operation" : "update", "source" : feedUrl, "feedId" : itemData.id}
                            thinXmlHttpRequestWraper.loadXml(feedUrl)
                        }
                    }
                } else {
                    /* Default value for faster development :)
                     * URL in comment - my Russian site, on which I tested program.
                     */
                    var feedUrl = (tfFeedUrl.text == "")?  "http://news.yahoo.com/rss/" : tfFeedUrl.text // "http://qtrosapps.hostenko.com/feed/"

                    var dbResult = DB2.addFeed(feedUrl, feedUrl)
                    if (dbResult.exist) {
                        return
                    }

                    // TODO mb insert 0
                    var modelData = {"source": feedUrl,
                                    "title": feedUrl,
                                    "description": "",
                                    "id" : dbResult.feedId,
                                    "status": '0'}
                    feedListModel.append(modelData)

                    pageItself.currentTask = {"operation" : "append", "source" : feedUrl,
                                              "feedId" : dbResult.feedId}
                    thinXmlHttpRequestWraper.loadXml(feedUrl)
                }
            }

            onCancelClicked: {
                console.log("Canceled")
            }

            contentsWidth: feedList.width - units.gu(7)
            // contentsHeight: units.gu(11) // TODO BUG - doesn't works
            title: isForEdit? i18n.tr("Edit feed") : i18n.tr("Append feed")

            Column {
                spacing: units.gu(1)
                width: units.gu(30)
                anchors.centerIn: parent

                TextField {
                    objectName: "newFeedUrl"
                    id: tfFeedUrl
                    anchors.horizontalCenter: parent.horizontalCenter
                    width: parent.width
                    placeholderText: isForEdit? i18n.tr("Enter new name here...") : i18n.tr("Enter url here...")
                }

                Item {
                    width: parent.width
                    height: units.gu(5)

                    Label {
                        anchors {
                            left: parent.left
                            verticalCenter: parent.verticalCenter
                            leftMargin: units.gu(1)
                        }
                        text: i18n.tr("Show in inbox")
                    }

                    Switch {
                        id: swShowOnMainscreen
                        checked: true
                        anchors {
                            right: parent.right
                            verticalCenter: parent.verticalCenter
                            rightMargin: units.gu(1)
                        }
                    }
                }
            } // Column
        } // ComposerSheet
    } // Component popoverSheetComponent

    Label {
        id: lblEmptyBase
        text: i18n.tr("There are no RSS feeds to show")
        anchors.centerIn: parent
        visible: feedListModel.count == 0
    }

    ActivityIndicator {
        visible: pageItself.currentTask != null
        running: true


        anchors {
            horizontalCenter: parent.horizontalCenter
            top: lblEmptyBase.bottom
            topMargin: units.gu(2)
        }
    }

//    Row {

//        anchors {
//            bottom: parent.bottom
//            horizontalCenter: parent.horizontalCenter
//            margins: units.gu(2)
//        }
//        spacing: units.gu(2)

//        // Temporary buttons: I just need toolbar icons...
//        // And first time is much more faster than swipe to show toolbar.
//        Button {
//            id: btnAppendFeed

//            text: i18n.tr("Append")
//            width: units.gu(15)

//            onClicked: {
//                PopupUtils.open(popoverSheetComponent, btnAppendFeed)
//            }
//        } // Button
//    } // Row
} // Page
