/*
  license GPL v3 ...........

  description of this file:
  a delegate for the ListView in the mainView, each delegate shows a whole rss feed ;

  Architecture:
  Item
  {
    Column
    {
        ListView(for feed title)
        {
            Label
        }

        ListView(for feed content)
        {
            RssItemDelegate
        }
    }

    Component(delegate)

    XmlListModel(for feed content)

    XmlListModel(for feed title)
  }
*/

import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem

Item {
    id: rss_feed_delegate
//    width: 100
    height: isList ? units.gu(5) : units.gu(35)
//    clip: true

    property string rss_url: ""
    property bool isList: false

    signal item_click(int rss_index, string rss_model, string rss_title)

    Column
    {
        width: parent.width
//        height: units.gu(40)
        spacing: isList ? 0 : units.gu(1)

//        Label
//        {
//            fontSize: "large"
//            text: "Yahoo Top Stories"
//        }

        // a ListView to show the title
        ListView
        {
            id: rsstitle
            width: parent.width
            height: units.gu(4)
            interactive: false
            contentHeight: units.gu(4)
            model: rsstitle_model
            delegate: Label { fontSize: "large"; text: title}
            clip: true
        }

        // a listview to show the RSS content
        ListView
        {
            id: rss_listview
            width: parent.width
            height: isList ? 0 : units.gu(30)
            opacity: isList ? 0 : 1
            cacheBuffer: 90
            spacing: units.gu(2)
            boundsBehavior: Flickable.StopAtBounds
            orientation: ListView.Horizontal
            contentHeight: units.gu(30) * count
            model: rssmodel
            delegate: xmldelegate
        }
    }

    Component
    {
        id: xmldelegate

        RssItemDelegate
        {
            width: units.gu(30)
            height: rss_listview.height
            rss_title: model_title
            rss_description:
            {
                if (description.split("</a>").length > 1)
                    return description.split("</a>")[1].split("</p>")[0] ;
                else
                    return description ;
            }

            MouseArea
            {
                anchors.fill: parent
                onClicked:
                {
                    console.log("rss index : " + index)
                    item_click(index, rssmodel.source, rsstitle_model.get(0).title) ;
                }
            }

//            onClick:
//            {
//                console.log("rss index : " + index)
//                click(index, rssmodel.xml, rsstitle_model.get(0).title) ;
//            }
        }
    }

    /*
      the both two XmlListModel below use source to load a rss feed url,
      but this only used in demo, if offline functions implemented,
      an entire xml content will be pass to both two XmlListModel
    */

    // rss content
    XmlListModel
    {
        id: rssmodel
        source: rss_url
        query: "/rss/channel/item"

        XmlRole { name: "model_title"; query: "title/string()" }
        XmlRole { name: "description"; query: "description/string()" }
    }

    // rss title
    XmlListModel
    {
        id: rsstitle_model
        source: rss_url
        query: "/rss/channel"

        XmlRole { name: "title"; query: "title/string()" }
        XmlRole { name: "pubdate"; query: "pubDate/string()" }
    }
}
