/*
  license GPL v3 ...........

  description of this demo:
  This is a demo about viewing RSS feeds, no offline functions implemented yet;

  files:
  rssview.qml : main view of demo
  RssFeedDelegate.qml : a delegate for listvew in mosaic mode (listview in mainView)
  RssFeedPage.qml : an independent page for viewing one RSS feed
  RssItemDelegate.qml : a delegate for listview in RssFeedDelegate.qml

  Architecture:
  mainView
  {
    pageStack
    {
        tabs
        {
            tab(feeds)
            {
                listview
                {
                    RssFeedDelegate
                    {
                        RssItemDelegate
                    }
                }
            }
        }

        RssFeedPage
    }

    Component(delegate)

    ListModel(RSS feeds)
  }
*/

import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1

/*!
    \brief MainView with Tabs element.
           First Tab has a single Label and
           second Tab has a single ToolbarAction.
*/

MainView {
    // objectName for functional testing purposes (autopilot-qt5)
    id: mainView
    objectName: "mainView"
    applicationName: "ubuntutab0"
    
    width: units.gu(50)
    height: units.gu(75)

    // a property for judging list mode or mosaic mode
    property bool isListMode: false

    // this PageStack use for pushing the independent RssFeedPage after user click an RSS item
    PageStack {
        id: pageStack
        anchors.fill: parent
//        Component.onCompleted:
//        {
//            push(tabs);                      // use tabs as first page
////            pageStack.__showHeader = false ; // hide the pageStack header
//        }

        Tabs {
            id: tabs
            anchors.fill: parent

            // First tab begins here
            Tab {
                objectName: "Tab1"

                title: i18n.tr("RSS2 Reader")
                clip: true

                                                   // main page for RSS feeds viewing
                page: FeedListPage {

                }
//                    Page {
//                    id: page_mosaic
//                    anchors.margins: units.gu(2)
////                    visible: false
//                    //                clip: true

//                    tools: ToolbarActions {
//                        Action {
//                            objectName: "action"

//                            iconSource: Qt.resolvedUrl("avatar.png")
//                            text: i18n.tr("Change mode")

//                            onTriggered: {
//                                isListMode = isListMode ? false : true ;    // switch mode for mosaic or list
//                            }
//                        }
//                    }

//                    // ListView for RSS feeds
//                    ListView
//                    {
//                        id: feed_view
//                        //                    anchors.left: parent.left
//                        //                    anchors.leftMargin: units.gu(1)
//                        anchors.fill: parent
//                        cacheBuffer: 90
//                        spacing: units.gu(2)
//                        contentHeight: isListMode ? units.gu(7) * count : units.gu(37) * count
//                        model: rssFeeds
//                        delegate: rss_feed_delegate
//                        //                    clip: true
//                    }
                }

            }

//                    // Second tab begins here
//                    Tab {
//                        objectName: "Tab2"

//                        title: i18n.tr("Optional Screen")
//                        page: Page {
//                            anchors.margins: units.gu(2)

//                            tools: ToolbarActions {
//                                Action {
//                                    objectName: "action"

//                                    iconSource: Qt.resolvedUrl("avatar.png")
//                                    text: i18n.tr("Tap me!")

//                                    onTriggered: {
//                                        label.text = i18n.tr("Toolbar tapped")
//                                    }
//                                }
//                            }

//                            Column {
//                                anchors.centerIn: parent
//                                Label {
//                                    id: label
//                                    objectName: "label"

//                                    text: i18n.tr("Swipe from bottom to up to reveal the toolbar.")
//                                }
//                            }
//                        }
//                    }
//        }

        // independent page for viewing one RSS feed
//        RssFeedPage
//        {
//            id: feed_page
//            visible: false
//        }
    }

//    Component
//    {
//        id: xmldelegate

//        RssItemDelegate
//        {
//            width: units.gu(30)
//            height: xmlview.height
//            rss_title: title
//            rss_description:
//            {
//                if (description.split("</a>").length > 1)
//                    return description.split("</a>")[1].split("</p>")[0] ;
//                else
//                    return description ;
//            }
//        }
//    }

//    XmlListModel
//    {
//        id: xmlmodel
//        source: "http://news.yahoo.com/rss/topstories"
//        query: "/rss/channel/item"

//        XmlRole { name: "title"; query: "title/string()" }
//        XmlRole { name: "description"; query: "description/string()" }
//    }

    // delegate for listview
    Component
    {
        id: rss_feed_delegate

        RssFeedDelegate
        {
            width: parent.width
            rss_url: feed
            isList: isListMode

            ////////////////////////////////////////////       click an RSS item to push RssFeedPage to stack
            onItem_click:
            {
                feed_page.rssmodel_url = rss_model ;
                feed_page.rss_index = rss_index ;
                feed_page.rsstitle = rss_title ;
                pageStack.push(feed_page);
            }
        }
    }

    // RSS feeds example
    ListModel {
        id: rssFeeds

        ListElement { name: "Top Stories"; feed: "http://news.yahoo.com/rss/topstories" }
        ListElement { name: "World"; feed: "http://news.yahoo.com/rss/world" }
        ListElement { name: "Europe"; feed: "http://news.yahoo.com/rss/europe" }
        ListElement { name: "Oceania"; feed: "http://news.yahoo.com/rss/oceania" }
        ListElement { name: "U.S. National"; feed: "http://news.yahoo.com/rss/us" }
        ListElement { name: "Politics"; feed: "http://news.yahoo.com/rss/linux" }
        ListElement { name: "Business"; feed: "http://news.yahoo.com/rss/business" }
        ListElement { name: "Technology"; feed: "http://news.yahoo.com/rss/tech" }
        ListElement { name: "Entertainment"; feed: "http://news.yahoo.com/rss/entertainment" }
        ListElement { name: "Health"; feed: "http://news.yahoo.com/rss/health" }
        ListElement { name: "Science"; feed: "http://news.yahoo.com/rss/science" }
        ListElement { name: "Sports"; feed: "http://news.yahoo.com/rss/sports" }
    }

    // a popup to show options
    Component {
        id: popoverComponent

        Popover {
            id: popover
            z: 10

            Component.onCompleted: console.log("popup")

            Column {
                id: containerLayout
//                anchors.fill: parent
                anchors {
                    left: parent.left
                    top: parent.top
                    right: parent.right
                }

                ListItem.Standard {
                    text: "Hot"
//                    selected: articleList.filter == 'hot'
                    onClicked: {
//                        articleList.filter = 'hot'
                        PopupUtils.close(popover)
                    }
                }
                ListItem.Standard {
                    text: "New"
//                    selected: articleList.filter == 'new'
                    onClicked: {
//                        articleList.filter = 'new'
                        PopupUtils.close(popover)
                    }
                }

            }
        } // Popover
    } // Popover component
}
