# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2013 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""RSS Reader app autopilot tests."""

from __future__ import absolute_import

from autopilot.matchers import Eventually
from testtools.matchers import Equals, NotEquals
from autopilot.introspection.dbus import StateNotFoundError

from ubuntu_rssreader_app.tests import RssreaderTestCase
from time import sleep

class TestMainWindow(RssreaderTestCase):

    def setUp(self):
        super(TestMainWindow, self).setUp()
        self.assertThat(
            self.ubuntusdk.get_qml_view().visible, Eventually(Equals(True)))

    def tearDown(self):
        super(TestMainWindow, self).tearDown()

    def test_add_new_feed(self):
        self.insert_check_canonical_feed()
        self.remove_check_canonical_feed()

    def test_view_feeds(self):
        #add Canonical feed
        self.insert_check_canonical_feed()

        #-----> view all news:
        #click list all news list item
        list_all_news = self.main_window.get_list_all_news()

        #does it exist?
        self.assertThat(list_all_news, NotEquals(None))

        #let's click it
        self.mouse.move_to_object(list_all_news)
        self.mouse.click()

        #verify list all news page is open
        view = lambda: self.ubuntusdk.get_qml_view()
        self.assertThat(view, Eventually(Equals("ArticleListPage.qml")))
        self._click_back_Toolbar_Button()

        #-----> view individual feed:
        #click Canonical feed item
        self.select_and_click_check_canonical_feed()
        self._click_back_Toolbar_Button()

        self.remove_check_canonical_feed()

    def select_and_click_check_canonical_feed(self):
        canonical_feed = lambda: self.main_window.get_canonical_feed()

        #does it exist?
        self.assertThat(canonical_feed, Eventually(NotEquals(None)))

        #click on Canonical feed
        self.mouse.move_to_object(canonical_feed())
        self.mouse.click()

        #verify list all news page is open
        view = lambda: self.ubuntusdk.get_qml_view()
        self.assertThat(view, Eventually(Equals("ArticleListPage.qml")))

    def insert_check_canonical_feed(self):
        #click on add feed button
        self.ubuntusdk.click_toolbar_button('Add feed')

        #type feed in input field
        new_Feed_Url = lambda: self.main_window.get_new_event_name_input_box()
        self.assertThat(new_Feed_Url, Eventually(NotEquals(None)))
        self.pointing_device.click_object(new_Feed_Url())
        self.keyboard.type("http://www.canonical.com/rss.xml")

        #click confirm button to save RSS
        confirm_btn = self.main_window.get_confirm_button()

        #does it exist?
        self.assertThat(confirm_btn, NotEquals(None))

        #let's click it
        self.mouse.move_to_object(confirm_btn)
        self.mouse.click()

        #verify Feed list page is open
        view = lambda: self.ubuntusdk.get_qml_view()
        self.assertThat(view, Eventually(Equals("FeedListPage.qml")))

        #check that feed is updated, not empty
        #canonical_feed_before = "http://www.canonical.com/rss.xml"
        #self.assertThat(lambda: self._check_canonical_feed(), Eventually(Equals(canonical_feed_before)))

        #verify feed has been added
        self.assertThat(lambda: self._check_canonical_feed(), Eventually(Equals("Canonical")))

    def _check_canonical_feed(self):
        try:
            return self.main_window.get_canonical_feed().text
        except StateNotFoundError:
            return None

    def remove_check_canonical_feed(self):
        canonical_feed = self.main_window.get_canonical_feed()

        #does it exist?
        self.assertThat(canonical_feed, NotEquals(None))

        #verify list all news page is open
        view = lambda: self.ubuntusdk.get_qml_view()
        self.assertThat(view, Eventually(Equals("ArticleListPage.qml")))

        #let's click it
        self.mouse.move_to_object(canonical_feed)
        self.mouse.click_object(canonical_feed, button=1, press_duration=4)

        #select remove feed from popover context menu
        remove_in_context_menu = self.main_window.get_remove_form_popOver()

        #let's click it
        self.mouse.move_to_object(remove_in_context_menu)
        self.mouse.click()
        self.assertThat(lambda: self.main_window.get_canonical_feed(), Eventually(Equals(None), timeout=30))

    #this should be deprecated when the move is made to the new toolbars
    def _click_back_Toolbar_Button(self):
        #toggle toolbar
        self.ubuntusdk.toggle_toolbar()
        #click on back button
        backTBButton = self.app.select_single('Button', text='Back')
        #let's click it
        self.mouse.move_to_object(backTBButton)
        self.mouse.click()
