import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem


/*!
    \brief MainView with Tabs element.
           First Tab has a single Label and
           second Tab has a single ToolbarAction.
*/

import "databasemodule.js" as DB

MainView {
    // objectName for functional testing purposes (autopilot-qt5)
    id: mainView
    objectName: "rssreader"
    applicationName: "ubuntu-rssreader-app"
    
    width: units.gu(50)
    height: units.gu(75)

    PageStack {
        id: mainPageStack

        anchors.fill: parent

        Component.onCompleted: {
            push(applicationTabs)
        }

        Tabs {
            id: applicationTabs
            anchors.fill: parent

            Tab {
                id: feedListTab
                objectName: "Tab1"

                title: i18n.tr("RSS Reader")
                clip: true

                page: FeedListPage {
                    id: feedListPage

                    onFeedSelected: {
                        articleListPage.loadArticles(params)
                        applicationTabs.selectedTabIndex = 1
                    }
                }
            }

            Tab {
                id: entryListTab
                objectName: "Tab2"

                title: i18n.tr("Articles")
                clip: true

                page: ArticleListPage {
                    id: articleListPage

                    onBackRequst: {
                        applicationTabs.selectedTabIndex = 0 // TODO REMEMBER PREV
                    }

                    onArticleSelected: {
                        mainPageStack.push(singleEntryPage)
                        singleEntryPage.content.loadHtml(data)
                    }
                }
            }

            Tab {
                id: tagsListTab
                objectName: "Tab3"

                title: i18n.tr("Tags")
                clip: true

                page: TagsListPage {
                    id: tagsListPage

                }
            }
        }

        SingleEntryPage {
            id: singleEntryPage
            visible: false

            onBackRequest: {
                // pop() happens automatically by std "back" action.
                // mainPageStack.pop()
            }
        }
    }



    // RSS feeds example
    ListModel {
        id: rssFeeds

        ListElement { name: "Top Stories"; feed: "http://news.yahoo.com/rss/topstories" }
        ListElement { name: "World"; feed: "http://news.yahoo.com/rss/world" }
        ListElement { name: "Europe"; feed: "http://news.yahoo.com/rss/europe" }
        ListElement { name: "Oceania"; feed: "http://news.yahoo.com/rss/oceania" }
        ListElement { name: "U.S. National"; feed: "http://news.yahoo.com/rss/us" }
        ListElement { name: "Politics"; feed: "http://news.yahoo.com/rss/linux" }
        ListElement { name: "Business"; feed: "http://news.yahoo.com/rss/business" }
        ListElement { name: "Technology"; feed: "http://news.yahoo.com/rss/tech" }
        ListElement { name: "Entertainment"; feed: "http://news.yahoo.com/rss/entertainment" }
        ListElement { name: "Health"; feed: "http://news.yahoo.com/rss/health" }
        ListElement { name: "Science"; feed: "http://news.yahoo.com/rss/science" }
        ListElement { name: "Sports"; feed: "http://news.yahoo.com/rss/sports" }
    }
}
